class Gazetteer:

    __data: set
    __name: str

    """
    A constructor for a specific gazetteer. The constructor takes name of the gazetteer and file name of the
    gazetteer as input, reads the gazetteer from the input file.

    PARAMETERS
    ----------
    name : str
        Name of the gazetteer. This name will be used in programming to separate different gazetteers.
    fileName : str
        File name of the gazetteer data.
    """
    def __init__(self, name: str, fileName: str):
        self.__name = name
        self.__data = set()
        inputFile = open(fileName, "r", encoding="utf8")
        lines = inputFile.readlines()
        for line in lines:
            self.__data.add(line)

    """
    Accessor method for the name of the gazetteer.

    RETURNS
    -------
    str
        Name of the gazetteer.
    """
    def getName(self) -> str:
        return self.__name

    """
    The most important method in Gazetteer class. Checks if the given word exists in the gazetteer. The check
    is done in lowercase form.

    PARAMETERS
    ----------
    word : str
        Word to be search in Gazetteer.

    RETURNS
    -------
    bool
        True if the word is in the Gazetteer, False otherwise.
    """
    def contains(self, word: str) -> bool:
        return word.lower() in self.__data



from Dictionary.Word import Word

from NamedEntityRecognition.NamedEntityType import NamedEntityType


class NamedEntityWord(Word):

    __namedEntityType: NamedEntityType

    """
    A constructor of NamedEntityWord which takes name and nameEntityType as input and sets the corresponding attributes

    PARAMETERS
    ----------
    name : str
        Name of the word
    namedEntityType : NamedEntityType
        NamedEntityType of the word
    """
    def __init__(self, name: str, namedEntityType: NamedEntityType):
        super().__init__(name)
        self.__namedEntityType = namedEntityType

    """
    Accessor method for namedEntityType attribute.

    RETURNS
    -------
    NamedEntityType
        namedEntityType of the word.
    """
    def getNamedEntityType(self) -> NamedEntityType:
        return self.__namedEntityType



from __future__ import annotations
from enum import Enum, auto

"""
Enumerator class for Named Entity types.
"""

class NamedEntityType(Enum):
    NONE = auto()
    PERSON = auto()
    ORGANIZATION = auto()
    LOCATION = auto()
    TIME = auto()
    MONEY = auto()

    """
    Static function to convert a string entity type to NamedEntityType type.

    PARAMETERS
    ----------
    entityType : str
        Entity type in string form
    
    RETURNS
    -------
    NamedEntityType
        Entity type in NamedEntityType form
    """
    @staticmethod
    def getNamedEntityType(entityType: str) -> NamedEntityType:
        for type in NamedEntityType:
            if entityType == type.name:
                return type
        return NamedEntityType.NONE

    """
    Static function to convert a NamedEntityType to string form.
    
    PARAMETERS
    ----------
    entityType : NamedEntityType
        Entity type in NamedEntityType form
        
    RETURNS
    -------
    str
        Entity type in string form
    """
    @staticmethod
    def getNamedEntityString(namedEntityType: NamedEntityType) -> str:
        if namedEntityType is None:
            return "NONE"
        return namedEntityType.name



from Corpus.Corpus import Corpus

from NamedEntityRecognition.NamedEntitySentence import NamedEntitySentence


class NERCorpus(Corpus):

    """
    Another constructor of NERCorpus which takes a fileName of the corpus as an input, reads the
    corpus from that file.

    PARAMETERS
    ----------
    fileName : str
        Name of the corpus file.
    """
    def __init__(self, fileName=None):
        super().__init__()
        if fileName is not None:
            inputFile = open(fileName, "r", encoding="utf8")
            lines = inputFile.readlines()
            for line in lines:
                self.addSentence(NamedEntitySentence(line))

    """
    addSentence adds a new sentence to the sentences list

    PARAMETERS
    ----------
    s : Sentence
        Sentence to be added.
    """
    def addSentence(self, s: NamedEntitySentence):
        self.sentences.append(s)
